import { createLog } from '../log';
import type { Logger } from 'winston';

import { SerialPort } from 'serialport';
import { ReadlineParser } from '@serialport/parser-readline';

interface PortInfo {
    path: string;
    manufacturer: string | undefined;
    serialNumber: string | undefined;
    pnpId: string | undefined;
    locationId: string | undefined;
    productId: string | undefined;
    vendorId: string | undefined;
}

export class Camera {
	private log : Logger;
	private parser : ReadlineParser; 
	private ready : boolean = false;
	private serial : any = null;
	private baud : number = 57600;

	constructor () {
		this.log = createLog('camera');
		this.parser = new ReadlineParser({ delimiter: '\r\n' });
		this.begin();
	}

	private async begin () {
		let ports : string[] = [];
		let selected : boolean = false;
		try {
			ports = await this.enumerate();
		} catch (err) {
			this.log.error('Error calling enumerate()', err);
		}
		if (ports.length === 0) {
			this.log.warn('No USB camera devices connected');
			return;
		}
		for (let port of ports) {
			this.log.info(`Found USB serial device: ${port} ${selected ? '*' : ''}`);
			selected = false;
		}
		try {
			await this.connect(ports[0]);
		} catch (err) {
			this.log.error(`Error connecting to ${ports[0]}`, err);
		}
	}

	private filter (port : PortInfo) {
		if ((port.manufacturer + '').toLowerCase().indexOf('arduino') !== -1 ||
			(port.path + '').toLowerCase().indexOf('usbserial') !== -1 ||
			(port.path + '').toLowerCase().indexOf('usbmodem') !== -1 || 
			(port.path + '').toLowerCase().indexOf('ttyusb') !== -1) {
			return true;
		}
		return false;
	}

	private async enumerate () : Promise<string[]> {
		let listed : PortInfo[] = [];
		try {
			listed = await SerialPort.list();
		} catch (err) {
			this.log.error('Error listing serial ports', err);
		}
		return listed.filter(this.filter).map((port : PortInfo) => port.path);
	}

	private async connect (port : string) {
		try {
			this.serial = new SerialPort({
				path : port,
				autoOpen : false,
				baudRate: this.baud,
			})
		} catch (err) {
			this.log.error('Error creating SerialPort object', err);
		}
		if (this.serial !== null) {
			this.serial.pipe(this.parser);
			this.parser.on('data', this.onData.bind(this));
		}
		this.ready = true;
	}

	private onData (data : string) {
		this.log.info(`Data received: "${data}"`);
	}
}

module.exports = { Camera };